@@ -1264,7 +1264,7 @@ void cache_init(cache_back* cache,httrackp* opt) {
       }
     }
     if ((opt->debug>0) && (opt->log!=NULL)) {
-      HTS_LOG(opt,LOG_DEBUG); fprintf(opt->log,"Cache: size %d"LF, fsize(fconcat(OPT_GET_BUFF(opt), StringBuff(opt->path_log),"hts-cache/old.zip")));
+      HTS_LOG(opt,LOG_DEBUG); fprintf(opt->log,"Cache: size %d"LF, (int)fsize(fconcat(OPT_GET_BUFF(opt), StringBuff(opt->path_log),"hts-cache/old.zip")));
     }
 
     // charger index cache prcdent
