@@ -231,6 +231,7 @@ HTSEXT_API int hts_main2(int argc, char **argv, httrackp *opt) {
 
       /* Vrifier argv[] non vide */
       if (strnotempty(argv[na])) {
+		  assertf(strlen(argv[na]) < HTS_CDLMAXSIZE);
         
         /* Vrifier Commande (alias) */
         result=optalias_check(argc,(const char * const *)argv,na,
@@ -1785,11 +1786,17 @@ HTSEXT_API int hts_main2(int argc, char **argv, httrackp *opt) {
         
       }  else {  // URL/filters
 				char catbuff[CATBUFF_SIZE];
-        char BIGSTK tempo[1024];       
-        if (strnotempty(url)) strcatbuff(url," ");  // espace de sparation
-        strcpybuff(tempo,unescape_http_unharm(catbuff,argv[na],1));
-        escape_spc_url(tempo);
-        strcatbuff(url,tempo);
+        char BIGSTK tempo[CATBUFF_SIZE];
+		const int urlSize = (int) strlen(argv[na]);
+		const int capa = (int) ( strlen(url) + urlSize + 32 );
+		assertf(urlSize < HTS_URLMAXSIZE);
+		if (urlSize < HTS_URLMAXSIZE) {
+			ensureUrlCapacity(url, url_sz, capa);
+			if (strnotempty(url)) strcatbuff(url," ");  // espace de sparation
+			strcpybuff(tempo,unescape_http_unharm(catbuff,argv[na],1));
+			escape_spc_url(tempo);
+			strcatbuff(url,tempo);
+		}
       }  // if argv=- etc. 
       
     }  // for
