@@ -40,11 +40,11 @@ Please visit our Website: http://www.httrack.com
 #ifndef HTSNAME_DEFH
 #define HTSNAME_DEFH 
 
-#include "htscore.h"
+#include "htsglobal.h"
 
 #define DELAYED_EXT "delayed"
 #define IS_DELAYED_EXT(a) ( ((a) != NULL) && ((a)[0] != 0) && strendwith_(a, "." DELAYED_EXT) )
-static int strendwith_(const char* a, const char* b) {
+HTS_STATIC int strendwith_(const char* a, const char* b)  {
   int i, j;
   for(i = 0 ; a[i] != 0 ; i++);
   for(j = 0 ; b[j] != 0 ; j++);
@@ -55,14 +55,49 @@ static int strendwith_(const char* a, const char* b) {
   return (j == -1);
 }
 
-
 /* Library internal definictions */
 #ifdef HTS_INTERNAL_BYTECODE
+
+/* Forward definitions */
+#ifndef HTS_DEF_FWSTRUCT_httrackp
+#define HTS_DEF_FWSTRUCT_httrackp
+typedef struct httrackp httrackp;
+#endif
+#ifndef HTS_DEF_FWSTRUCT_lien_url
+#define HTS_DEF_FWSTRUCT_lien_url
+typedef struct lien_url lien_url;
+#endif
+#ifndef HTS_DEF_FWSTRUCT_struct_back
+#define HTS_DEF_FWSTRUCT_struct_back
+typedef struct struct_back struct_back;
+#endif
+#ifndef HTS_DEF_FWSTRUCT_cache_back
+#define HTS_DEF_FWSTRUCT_cache_back
+typedef struct cache_back cache_back;
+#endif
+#ifndef HTS_DEF_FWSTRUCT_hash_struct
+#define HTS_DEF_FWSTRUCT_hash_struct
+typedef struct hash_struct hash_struct;
+#endif
+#ifndef HTS_DEF_FWSTRUCT_lien_back
+#define HTS_DEF_FWSTRUCT_lien_back
+typedef struct lien_back lien_back;
+#endif
+
 // note: 'headers' can either be null, or incomplete (only r member filled)
-int url_savename(char* adr_complete,char* fil_complete,char* save,char* former_adr,char* former_fil,char* referer_adr,char* referer_fil,httrackp* opt,lien_url** liens,int lien_tot,struct_back* sback,cache_back* cache,hash_struct* hash,int ptr,int numero_passe,const lien_back* headers);
+int url_savename(char* adr_complete, char* fil_complete, char* save,
+								 char* former_adr, char* former_fil, 
+								 char* referer_adr, char* referer_fil, 
+								 httrackp* opt, 
+								 lien_url** liens, int lien_tot, 
+								 struct_back* sback, 
+								 cache_back* cache, 
+								 hash_struct* hash, 
+								 int ptr, int numero_passe, 
+								 const lien_back* headers);
 void standard_name(char* b,char* dot_pos,char* nom_pos,char* fil_complete,int short_ver);
 void url_savename_addstr(char* d,char* s);
-char* url_md5(char* fil_complete);
+char* url_md5(char* digest_buffer, char* fil_complete);
 #endif
 
 #endif
